@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use Config;
 
-our $VERSION = '2.000011';
+our $VERSION = '2.000012';
 $VERSION = eval $VERSION;
 
 BEGIN {
@@ -21,10 +21,25 @@ our $_ROOT = _WIN32 ? do {
   my $UNC = qr{[\\/]{2}[^\\/]+[\\/][^\\/]+};
   qr{^(?:$UNC|[A-Za-z]:|)$_DIR_SPLIT};
 } : qr{^/};
-our ($_PERL) = $^X =~ /(.+)/; # $^X is internal how could it be tainted?!
+our $_PERL;
 
 sub _cwd {
   my $drive = shift;
+  if (!$_PERL) {
+    ($_PERL) = $^X =~ /(.+)/; # $^X is internal how could it be tainted?!
+    if (_is_abs($_PERL)) {
+    }
+    elsif (-x $Config{perlpath}) {
+      $_PERL = $Config{perlpath};
+    }
+    else {
+      ($_PERL) =
+        map { /(.*)/ }
+        grep { -x $_ }
+        map { join($_DIR_JOIN, $_, $_PERL) }
+        split /\Q$Config{path_sep}\E/, $ENV{PATH};
+    }
+  }
   local @ENV{qw(PATH IFS CDPATH ENV BASH_ENV)};
   my $cmd = $drive ? "eval { Cwd::getdcwd(q($drive)) }"
                    : 'getcwd';
@@ -49,12 +64,20 @@ sub _catdir {
   }
 }
 
+sub _is_abs {
+  if (_USE_FSPEC) {
+    require File::Spec;
+    File::Spec->file_name_is_absolute($_[0]);
+  }
+  else {
+    $_[0] =~ $_ROOT;
+  }
+}
+
 sub _rel2abs {
   my ($dir, $base) = @_;
   return $dir
-    if (_USE_FSPEC && require File::Spec)
-      ? File::Spec->file_name_is_absolute($dir)
-      : $dir =~ $_ROOT;
+    if _is_abs($dir);
 
   $base = _WIN32 && $dir =~ s/^([A-Za-z]:)// ? _cwd("$1")
         : $base                              ? $base
@@ -334,6 +357,18 @@ sub build_environment_vars_for {
   my $self = $_[0]->new->activate($_[1]);
   $self->build_environment_vars;
 }
+sub build_activate_environment_vars_for {
+  my $self = $_[0]->new->activate($_[1]);
+  $self->build_environment_vars;
+}
+sub build_deactivate_environment_vars_for {
+  my $self = $_[0]->new->deactivate($_[1]);
+  $self->build_environment_vars;
+}
+sub build_deact_all_environment_vars_for {
+  my $self = $_[0]->new->deactivate_all;
+  $self->build_environment_vars;
+}
 sub build_environment_vars {
   my $self = shift;
   (
@@ -411,14 +446,6 @@ sub environment_vars_string {
         && ${$value->[0]} eq $name) {
       next;
     }
-    if (
-        !ref $value
-        and defined $value
-          ? (defined $ENV{$name} && $value eq $ENV{$name})
-          : !defined $ENV{$name}
-    ) {
-      next;
-    }
     $out .= $self->$build_method($name, $value);
   }
   my $wrap_method = "wrap_${shelltype}_output";
@@ -501,7 +528,7 @@ sub wrap_powershell_output {
 
 sub build_fish_env_declaration {
   my ($class, $name, $args) = @_;
-  my $value = $class->_interpolate($args, '$%s', '"', '\\%s');
+  my $value = $class->_interpolate($args, '$%s', qr/[" ]/, '\\%s');
   if (!defined $value) {
     return qq{set -e $name;\n};
   }
